@@ -34,10 +34,10 @@ static const char *getRegisterName(unsigned RegNo);
 #endif
 
 static void printOperand(MCInst *MI, unsigned OpNo, SStream *O);
-static void printInstruction(MCInst *MI, SStream *O, const MCRegisterInfo *MRI);
+static void printInstruction(MCInst *MI, SStream *O);
 static void printAbsBranchOperand(MCInst *MI, unsigned OpNo, SStream *O);
-static char *printAliasInstr(MCInst *MI, SStream *OS, void *info);
-static char *printAliasInstrEx(MCInst *MI, SStream *OS, void *info);
+static char *printAliasInstr(MCInst *MI, SStream *OS, MCRegisterInfo *MRI);
+static char *printAliasBcc(MCInst *MI, SStream *OS, void *info);
 static void printCustomAliasOperand(MCInst *MI, unsigned OpIdx,
 		unsigned PrintMethodIdx, SStream *OS);
 
@@ -80,17 +80,328 @@ void PPC_post_printer(csh ud, cs_insn *insn, char *insn_asm, MCInst *mci)
 		return;
 
 	// check if this insn has branch hint
-	if (strrchr(insn_asm, '+') != NULL && !strstr(insn_asm, ".+")) {
+	if (strrchr(insn->mnemonic, '+') != NULL && !strstr(insn_asm, ".+")) {
 		insn->detail->ppc.bh = PPC_BH_PLUS;
-	} else if (strrchr(insn_asm, '-') != NULL) {
+	} else if (strrchr(insn->mnemonic, '-') != NULL) {
 		insn->detail->ppc.bh = PPC_BH_MINUS;
 	}
+
+	if (strrchr(insn->mnemonic, '.') != NULL) {
+		insn->detail->ppc.update_cr0 = true;
+	}
 }
 
 #define GET_INSTRINFO_ENUM
 #include "PPCGenInstrInfo.inc"
 
-static int isBOCTRBranch(unsigned int op)
+#define GET_REGINFO_ENUM
+#include "PPCGenRegisterInfo.inc"
+
+static void op_addBC(MCInst *MI, unsigned int bc)
+{
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.bc = (ppc_bc)bc;
+	}
+}
+
+#define CREQ (0)
+#define CRGT (1)
+#define CRLT (2)
+#define CRUN (3)
+
+static int getBICRCond(int bi)
+{
+	return (bi - PPC_CR0EQ) >> 3;
+}
+
+static int getBICR(int bi)
+{
+	return ((bi - PPC_CR0EQ) & 7) + PPC_CR0;
+}
+
+static void op_addReg(MCInst *MI, unsigned int reg)
+{
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_REG;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].reg = reg;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+static void add_CRxx(MCInst *MI, ppc_reg reg)
+{
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_REG;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].reg = reg;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+static char *printAliasBcc(MCInst *MI, SStream *OS, void *info)
+{
+#define GETREGCLASS_CONTAIN(_class, _reg) MCRegisterClass_contains(MCRegisterInfo_getRegClass(MRI, _class), MCOperand_getReg(MCInst_getOperand(MI, _reg)))
+	SStream ss;
+	const char *opCode;
+	char *tmp, *AsmMnem, *AsmOps, *c;
+	int OpIdx, PrintMethodIdx;
+	int decCtr = false, needComma = false;
+	MCRegisterInfo *MRI = (MCRegisterInfo *)info;
+
+	SStream_Init(&ss);
+
+	switch (MCInst_getOpcode(MI)) {
+		default: return NULL;
+		case PPC_gBC:
+				 opCode = "b%s";
+				 break;
+		case PPC_gBCA:
+				 opCode = "b%sa";
+				 break;
+		case PPC_gBCCTR:
+				 opCode = "b%sctr";
+				 break;
+		case PPC_gBCCTRL:
+				 opCode = "b%sctrl";
+				 break;
+		case PPC_gBCL:
+				 opCode = "b%sl";
+				 break;
+		case PPC_gBCLA:
+				 opCode = "b%sla";
+				 break;
+		case PPC_gBCLR:
+				 opCode = "b%slr";
+				 break;
+		case PPC_gBCLRL:
+				 opCode = "b%slrl";
+				 break;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 0) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 1)) {
+		SStream_concat(&ss, opCode, "dnzf");
+		decCtr = true;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 2) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 3)) {
+		SStream_concat(&ss, opCode, "dzf");
+		decCtr = true;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 4) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 7) &&
+			MCOperand_isReg(MCInst_getOperand(MI, 1)) &&
+			GETREGCLASS_CONTAIN(PPC_CRBITRCRegClassID, 1)) {
+		int cr = getBICRCond(MCOperand_getReg(MCInst_getOperand(MI, 1)));
+
+		switch(cr) {
+			case CREQ:
+				SStream_concat(&ss, opCode, "ne");
+				break;
+			case CRGT:
+				SStream_concat(&ss, opCode, "le");
+				break;
+			case CRLT:
+				SStream_concat(&ss, opCode, "ge");
+				break;
+			case CRUN:
+				SStream_concat(&ss, opCode, "ns");
+				break;
+		}
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 6)
+			SStream_concat0(&ss, "-");
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 7)
+			SStream_concat0(&ss, "+");
+
+		decCtr = false;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 8) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 9)) {
+		SStream_concat(&ss, opCode, "dnzt");
+		decCtr = true;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 10) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 11)) {
+		SStream_concat(&ss, opCode, "dzt");
+		decCtr = true;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 12) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 15) &&
+			MCOperand_isReg(MCInst_getOperand(MI, 1)) &&
+			GETREGCLASS_CONTAIN(PPC_CRBITRCRegClassID, 1)) {
+		int cr = getBICRCond(MCOperand_getReg(MCInst_getOperand(MI, 1)));
+
+		switch(cr) {
+			case CREQ:
+				SStream_concat(&ss, opCode, "eq");
+				break;
+			case CRGT:
+				SStream_concat(&ss, opCode, "gt");
+				break;
+			case CRLT:
+				SStream_concat(&ss, opCode, "lt");
+				break;
+			case CRUN:
+				SStream_concat(&ss, opCode, "so");
+				break;
+		}
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 14)
+			SStream_concat0(&ss, "-");
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 15)
+			SStream_concat0(&ss, "+");
+
+		decCtr = false;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			((MCOperand_getImm(MCInst_getOperand(MI, 0)) & 0x12)== 16)) {
+		SStream_concat(&ss, opCode, "dnz");
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 24)
+			SStream_concat0(&ss, "-");
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 25)
+			SStream_concat0(&ss, "+");
+
+		needComma = false;
+	}
+
+	if (MCInst_getNumOperands(MI) == 3 &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			((MCOperand_getImm(MCInst_getOperand(MI, 0)) & 0x12)== 18)) {
+		SStream_concat(&ss, opCode, "dz");
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 26)
+			SStream_concat0(&ss, "-");
+
+		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 27)
+			SStream_concat0(&ss, "+");
+
+		needComma = false;
+	}
+
+	if (MCOperand_isReg(MCInst_getOperand(MI, 1)) &&
+			GETREGCLASS_CONTAIN(PPC_CRBITRCRegClassID, 1) &&
+			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
+			(MCOperand_getImm(MCInst_getOperand(MI, 0)) < 16)) {
+		int cr = getBICR(MCOperand_getReg(MCInst_getOperand(MI, 1)));
+
+		if (decCtr) {
+			int cd;
+			needComma = true;
+			SStream_concat0(&ss, " ");
+
+			if (cr > PPC_CR0) {
+				SStream_concat(&ss, "4*cr%d+", cr - PPC_CR0);
+			}
+
+			cd = getBICRCond(MCOperand_getReg(MCInst_getOperand(MI, 1)));
+			switch(cd) {
+				case CREQ:
+					SStream_concat0(&ss, "eq");
+					if (cr <= PPC_CR0)
+						add_CRxx(MI, PPC_REG_CR0EQ);
+					op_addBC(MI, PPC_BC_EQ);
+					break;
+				case CRGT:
+					SStream_concat0(&ss, "gt");
+					if (cr <= PPC_CR0)
+						add_CRxx(MI, PPC_REG_CR0GT);
+					op_addBC(MI, PPC_BC_GT);
+					break;
+				case CRLT:
+					SStream_concat0(&ss, "lt");
+					if (cr <= PPC_CR0)
+						add_CRxx(MI, PPC_REG_CR0LT);
+					op_addBC(MI, PPC_BC_LT);
+					break;
+				case CRUN:
+					SStream_concat0(&ss, "so");
+					if (cr <= PPC_CR0)
+						add_CRxx(MI, PPC_REG_CR0UN);
+					op_addBC(MI, PPC_BC_SO);
+					break;
+			}
+
+			if (cr > PPC_CR0) {
+				if (MI->csh->detail) {
+					MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_REG;
+					MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].reg = MCOperand_getReg(MCInst_getOperand(MI, 1));
+					MI->flat_insn->detail->ppc.op_count++;
+				}
+			}
+		} else {
+			if (cr > PPC_CR0) {
+				needComma = true;
+				SStream_concat(&ss, " cr%d", cr - PPC_CR0);
+				op_addReg(MI, PPC_REG_CR0 + cr - PPC_CR0);
+			}
+		}
+	}
+
+	if (MCOperand_isImm(MCInst_getOperand(MI, 2)) &&
+			MCOperand_getImm(MCInst_getOperand(MI, 2)) != 0) {
+		if (needComma)
+			SStream_concat0(&ss, ",");
+
+		SStream_concat0(&ss, " $\xFF\x03\x01");
+	}
+
+	tmp = cs_strdup(ss.buffer);
+	AsmMnem = tmp;
+	for(AsmOps = tmp; *AsmOps; AsmOps++) {
+		if (*AsmOps == ' ' || *AsmOps == '\t') {
+			*AsmOps = '\0';
+			AsmOps++;
+			break;
+		}
+	}
+
+	SStream_concat0(OS, AsmMnem);
+	if (*AsmOps) {
+		SStream_concat0(OS, "\t");
+		for (c = AsmOps; *c; c++) {
+			if (*c == '$') {
+				c += 1;
+				if (*c == (char)0xff) {
+					c += 1;
+					OpIdx = *c - 1;
+					c += 1;
+					PrintMethodIdx = *c - 1;
+					printCustomAliasOperand(MI, OpIdx, PrintMethodIdx, OS);
+				} else
+					printOperand(MI, *c - 1, OS);
+			} else {
+				SStream_concat1(OS, *c);
+			}
+		}
+	}
+
+	return tmp;
+}
+
+static bool isBOCTRBranch(unsigned int op)
 {
 	return ((op >= PPC_BDNZ) && (op <= PPC_BDZp));
 }
@@ -98,31 +409,36 @@ static int isBOCTRBranch(unsigned int op)
 void PPC_printInst(MCInst *MI, SStream *O, void *Info)
 {
 	char *mnem;
+	unsigned int opcode = MCInst_getOpcode(MI);
+	memset(O->buffer, 0, sizeof(O->buffer));
+
+	// printf("opcode = %u\n", opcode);
 
 	// Check for slwi/srwi mnemonics.
-	if (MCInst_getOpcode(MI) == PPC_RLWINM) {
+	if (opcode == PPC_RLWINM) {
 		unsigned char SH = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 2));
 		unsigned char MB = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 3));
 		unsigned char ME = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 4));
 		bool useSubstituteMnemonic = false;
 
-		if (SH <= 31 && MB == 0 && ME == (31-SH)) {
+		if (SH <= 31 && MB == 0 && ME == (31 - SH)) {
 			SStream_concat0(O, "slwi\t");
 			MCInst_setOpcodePub(MI, PPC_INS_SLWI);
 			useSubstituteMnemonic = true;
 		}
 
-		if (SH <= 31 && MB == (32-SH) && ME == 31) {
+		if (SH <= 31 && MB == (32 - SH) && ME == 31) {
 			SStream_concat0(O, "srwi\t");
 			MCInst_setOpcodePub(MI, PPC_INS_SRWI);
 			useSubstituteMnemonic = true;
-			SH = 32-SH;
+			SH = 32 - SH;
 		}
 
 		if (useSubstituteMnemonic) {
 			printOperand(MI, 0, O);
 			SStream_concat0(O, ", ");
 			printOperand(MI, 1, O);
+
 			if (SH > HEX_THRESHOLD)
 				SStream_concat(O, ", 0x%x", (unsigned int)SH);
 			else
@@ -140,40 +456,152 @@ void PPC_printInst(MCInst *MI, SStream *O, void *Info)
 		}
 	}
 
-	if ((MCInst_getOpcode(MI) == PPC_OR || MCInst_getOpcode(MI) == PPC_OR8) &&
+	if ((opcode == PPC_OR || opcode == PPC_OR8) &&
 			MCOperand_getReg(MCInst_getOperand(MI, 1)) == MCOperand_getReg(MCInst_getOperand(MI, 2))) {
 		SStream_concat0(O, "mr\t");
 		MCInst_setOpcodePub(MI, PPC_INS_MR);
+
 		printOperand(MI, 0, O);
 		SStream_concat0(O, ", ");
 		printOperand(MI, 1, O);
+
 		return;
 	}
 
-	if (MCInst_getOpcode(MI) == PPC_RLDICR) {
+	if (opcode == PPC_RLDICR ||
+			opcode == PPC_RLDICR_32) {
 		unsigned char SH = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 2));
 		unsigned char ME = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 3));
+
 		// rldicr RA, RS, SH, 63-SH == sldi RA, RS, SH
-		if (63-SH == ME) {
+		if (63 - SH == ME) {
 			SStream_concat0(O, "sldi\t");
 			MCInst_setOpcodePub(MI, PPC_INS_SLDI);
+
 			printOperand(MI, 0, O);
 			SStream_concat0(O, ", ");
 			printOperand(MI, 1, O);
+
 			if (SH > HEX_THRESHOLD)
 				SStream_concat(O, ", 0x%x", (unsigned int)SH);
 			else
 				SStream_concat(O, ", %u", (unsigned int)SH);
 
+			if (MI->csh->detail) {
+				cs_ppc *ppc = &MI->flat_insn->detail->ppc;
+
+				ppc->operands[ppc->op_count].type = PPC_OP_IMM;
+				ppc->operands[ppc->op_count].imm = SH;
+				++ppc->op_count;
+			}
+
+
 			return;
 		}
 	}
 
-	if ((MCInst_getOpcode(MI) == PPC_gBC)||(MCInst_getOpcode(MI) == PPC_gBCA)||
-			(MCInst_getOpcode(MI) == PPC_gBCL)||(MCInst_getOpcode(MI) == PPC_gBCLA)) {
+	// dcbt[st] is printed manually here because:
+	//  1. The assembly syntax is different between embedded and server targets
+	//  2. We must print the short mnemonics for TH == 0 because the
+	//     embedded/server syntax default will not be stable across assemblers
+	//  The syntax for dcbt is:
+	//    dcbt ra, rb, th [server]
+	//    dcbt th, ra, rb [embedded]
+	//  where th can be omitted when it is 0. dcbtst is the same.
+	if (opcode == PPC_DCBT || opcode == PPC_DCBTST) {
+		unsigned char TH = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 0));
+
+		SStream_concat0(O, "dcbt");
+		MCInst_setOpcodePub(MI, PPC_INS_DCBT);
+
+		if (opcode == PPC_DCBTST) {
+			SStream_concat0(O, "st");
+			MCInst_setOpcodePub(MI, PPC_INS_DCBTST);
+		}
+
+		if (TH == 16) {
+			SStream_concat0(O, "t");
+			MCInst_setOpcodePub(MI, PPC_INS_DCBTSTT);
+		}
+
+		SStream_concat0(O, "\t");
+
+		if (MI->csh->mode & CS_MODE_BOOKE && TH != 0 && TH != 16) {
+			if (TH > HEX_THRESHOLD)
+				SStream_concat(O, "0x%x, ", (unsigned int)TH);
+			else
+				SStream_concat(O, "%u, ", (unsigned int)TH);
+
+			if (MI->csh->detail) {
+				cs_ppc *ppc = &MI->flat_insn->detail->ppc;
+
+				ppc->operands[ppc->op_count].type = PPC_OP_IMM;
+				ppc->operands[ppc->op_count].imm = TH;
+				++ppc->op_count;
+			}
+		}
+
+		printOperand(MI, 1, O);
+		SStream_concat0(O, ", ");
+		printOperand(MI, 2, O);
+
+		if (!(MI->csh->mode & CS_MODE_BOOKE) && TH != 0 && TH != 16) {
+			if (TH > HEX_THRESHOLD)
+				SStream_concat(O, ", 0x%x", (unsigned int)TH);
+			else
+				SStream_concat(O, ", %u", (unsigned int)TH);
+
+			if (MI->csh->detail) {
+				cs_ppc *ppc = &MI->flat_insn->detail->ppc;
+
+				ppc->operands[ppc->op_count].type = PPC_OP_IMM;
+				ppc->operands[ppc->op_count].imm = TH;
+				++ppc->op_count;
+			}
+		}
+
+		return;
+	}
+
+	if (opcode == PPC_DCBF) {
+		unsigned char L = (unsigned char)MCOperand_getImm(MCInst_getOperand(MI, 0));
+
+		if (!L || L == 1 || L == 3) {
+			SStream_concat0(O, "dcbf");
+			MCInst_setOpcodePub(MI, PPC_INS_DCBF);
+
+			if (L == 1 || L == 3) {
+				SStream_concat0(O, "l");
+				MCInst_setOpcodePub(MI, PPC_INS_DCBFL);
+			}
+
+			if (L == 3) {
+				SStream_concat0(O, "p");
+				MCInst_setOpcodePub(MI, PPC_INS_DCBFLP);
+			}
+
+			SStream_concat0(O, "\t");
+
+			printOperand(MI, 1, O);
+			SStream_concat0(O, ", ");
+			printOperand(MI, 2, O);
+
+			return;
+		}
+	}
+
+	if (opcode == PPC_B || opcode == PPC_BA || opcode == PPC_BL ||
+			opcode == PPC_BLA) {
+		int64_t bd = MCOperand_getImm(MCInst_getOperand(MI, 0));
+		bd = SignExtend64(bd, 24);
+		MCOperand_setImm(MCInst_getOperand(MI, 0), bd);
+	}
+
+	if (opcode == PPC_gBC || opcode == PPC_gBCA || opcode == PPC_gBCL ||
+			opcode == PPC_gBCLA) {
 		int64_t bd = MCOperand_getImm(MCInst_getOperand(MI, 2));
 		bd = SignExtend64(bd, 14);
-		MCOperand_setImm(MCInst_getOperand(MI, 2),bd);
+		MCOperand_setImm(MCInst_getOperand(MI, 2), bd);
 	}
 
 	if (isBOCTRBranch(MCInst_getOpcode(MI))) {
@@ -184,15 +612,7 @@ void PPC_printInst(MCInst *MI, SStream *O, void *Info)
 		}
 	}
 
-	if ((MCInst_getOpcode(MI) == PPC_B)||(MCInst_getOpcode(MI) == PPC_BA)||
-			(MCInst_getOpcode(MI) == PPC_BL)||(MCInst_getOpcode(MI) == PPC_BLA)) {
-		int64_t bd = MCOperand_getImm(MCInst_getOperand(MI, 0));
-		bd = SignExtend64(bd, 24);
-		MCOperand_setImm(MCInst_getOperand(MI, 0),bd);
-	}
-
-	// consider our own alias instructions first
-	mnem = printAliasInstrEx(MI, O, Info);
+	mnem = printAliasBcc(MI, O, Info);
 	if (!mnem)
 		mnem = printAliasInstr(MI, O, Info);
 
@@ -202,22 +622,33 @@ void PPC_printInst(MCInst *MI, SStream *O, void *Info)
 			if (mnem[strlen(mnem) - 1] == '-' || mnem[strlen(mnem) - 1] == '+' || mnem[strlen(mnem) - 1] == '.')
 				mnem[strlen(mnem) - 1] = '\0';
 
-			MCInst_setOpcodePub(MI, PPC_map_insn(mnem));
+            MCInst_setOpcodePub(MI, PPC_map_insn(mnem));
 
-			if (MI->csh->detail) {
+            if (MI->csh->detail) {
 				struct ppc_alias alias;
 
 				if (PPC_alias_insn(mnem, &alias)) {
 					MI->flat_insn->detail->ppc.bc = (ppc_bc)alias.cc;
 				}
-			}
+            }
 		}
 
 		cs_mem_free(mnem);
 	} else
-		printInstruction(MI, O, NULL);
+		printInstruction(MI, O);
+
+	const char *mnem_end = strchr(O->buffer, ' ');
+	unsigned mnem_len = 0;
+	if (mnem_end)
+		mnem_len = mnem_end - O->buffer;
+	if (!mnem_end || mnem_len >= sizeof(MI->flat_insn->mnemonic))
+		mnem_len = sizeof(MI->flat_insn->mnemonic) - 1;
+
+	memset(MI->flat_insn->mnemonic, 0, sizeof(MI->flat_insn->mnemonic));
+	memcpy(MI->flat_insn->mnemonic, O->buffer, mnem_len);
 }
 
+// FIXME
 enum ppc_bc_hint {
 	PPC_BC_LT_MINUS = (0 << 5) | 14,
 	PPC_BC_LE_MINUS = (1 << 5) |  6,
@@ -237,6 +668,7 @@ enum ppc_bc_hint {
 	PPC_BC_NU_PLUS  = (3 << 5) |  7,
 };
 
+// FIXME
 // normalize CC to remove _MINUS & _PLUS
 static int cc_normalize(int cc)
 {
@@ -366,15 +798,52 @@ static void printPredicateOperand(MCInst *MI, unsigned OpNo,
 	printOperand(MI, OpNo + 1, O);
 }
 
+static void printATBitsAsHint(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned Code = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
+	if (Code == 2) {
+		SStream_concat0(O, "-");
+	} else if (Code == 3) {
+		SStream_concat0(O, "+");
+	}
+}
+
+static void printU1ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
+	// assert(Value <= 1 && "Invalid u1imm argument!");
+
+	printUInt32(O, Value);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Value;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
 static void printU2ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	unsigned int Value = (int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
 	//assert(Value <= 3 && "Invalid u2imm argument!");
 
-	if (Value > HEX_THRESHOLD)
-		SStream_concat(O, "0x%x", Value);
-	else
-		SStream_concat(O, "%u", Value);
+	printUInt32(O, Value);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Value;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+static void printU3ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned int Value = (int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+	//assert(Value <= 8 && "Invalid u3imm argument!");
+
+	printUInt32(O, Value);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -388,10 +857,7 @@ static void printU4ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	unsigned int Value = (int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
 	//assert(Value <= 15 && "Invalid u4imm argument!");
 
-	if (Value > HEX_THRESHOLD)
-		SStream_concat(O, "0x%x", Value);
-	else
-		SStream_concat(O, "%u", Value);
+	printUInt32(O, Value);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -417,6 +883,7 @@ static void printS5ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 static void printU5ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
 	//assert(Value <= 31 && "Invalid u5imm argument!");
 	printUInt32(O, Value);
 
@@ -430,6 +897,7 @@ static void printU5ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 static void printU6ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
 	//assert(Value <= 63 && "Invalid u6imm argument!");
 	printUInt32(O, Value);
 
@@ -440,16 +908,78 @@ static void printU6ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	}
 }
 
+static void printU7ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
+	//assert(Value <= 127 && "Invalid u7imm argument!");
+	printUInt32(O, Value);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Value;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+// Operands of BUILD_VECTOR are signed and we use this to print operands
+// of XXSPLTIB which are unsigned. So we simply truncate to 8 bits and
+// print as unsigned.
+static void printU8ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
+	printUInt32(O, Value);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Value;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+static void printU10ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
+	//assert(Value <= 1023 && "Invalid u10imm argument!");
+	printUInt32(O, Value);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Value;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+static void printS12ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	if (MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
+		int Imm = (int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+		Imm = SignExtend32(Imm, 12);
+
+		printInt32(O, Imm);
+
+		if (MI->csh->detail) {
+			if (MI->csh->doing_mem) {
+                MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].mem.disp = Imm;
+			} else {
+                MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+                MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Imm;
+                MI->flat_insn->detail->ppc.op_count++;
+            }
+		}
+	} else
+		printOperand(MI, OpNo, O);
+}
+
 static void printU12ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	unsigned short Value = (unsigned short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
 
 	// assert(Value <= 4095 && "Invalid u12imm argument!");
 
-	if (Value > HEX_THRESHOLD)
-		SStream_concat(O, "0x%x", Value);
-	else
-		SStream_concat(O, "%u", Value);
+	printUInt32(O, Value);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -459,48 +989,19 @@ static void printU12ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 }
 
 static void printS16ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
-{
-	if (MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
-		unsigned short Imm = (unsigned short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
-        if (Imm > HEX_THRESHOLD)
-            SStream_concat(O, "0x%x", Imm);
-        else
-            SStream_concat(O, "%u", Imm);
-
-		if (MI->csh->detail) {
-			MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
-			MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Imm;
-			MI->flat_insn->detail->ppc.op_count++;
-		}
-	} else
-		printOperand(MI, OpNo, O);
-}
-
-static void printS16ImmOperand_Mem(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	if (MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
 		short Imm = (short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
-
-		if (Imm >= 0) {
-			if (Imm > HEX_THRESHOLD)
-				SStream_concat(O, "0x%x", Imm);
-			else
-				SStream_concat(O, "%u", Imm);
-		} else {
-			if (Imm < -HEX_THRESHOLD)
-				SStream_concat(O, "-0x%x", -Imm);
-			else
-				SStream_concat(O, "-%u", -Imm);
-		}
+		printInt32(O, Imm);
 
 		if (MI->csh->detail) {
 			if (MI->csh->doing_mem) {
-				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].mem.disp = Imm;
+                MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].mem.disp = Imm;
 			} else {
-				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
-				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Imm;
-				MI->flat_insn->detail->ppc.op_count++;
-			}
+                MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+                MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Imm;
+                MI->flat_insn->detail->ppc.op_count++;
+            }
 		}
 	} else
 		printOperand(MI, OpNo, O);
@@ -510,10 +1011,7 @@ static void printU16ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	if (MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
 		unsigned short Imm = (unsigned short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
-		if (Imm > HEX_THRESHOLD)
-			SStream_concat(O, "0x%x", Imm);
-		else
-			SStream_concat(O, "%u", Imm);
+		printUInt32(O, Imm);
 
 		if (MI->csh->detail) {
 			MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -528,11 +1026,13 @@ static void printBranchOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	if (!MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
 		printOperand(MI, OpNo, O);
+
 		return;
 	}
 
 	// Branches can take an immediate operand.  This is used by the branch
 	// selection pass to print .+8, an eight byte displacement from the PC.
+	// O << ".+";
 	printAbsBranchOperand(MI, OpNo, O);
 }
 
@@ -542,16 +1042,18 @@ static void printAbsBranchOperand(MCInst *MI, unsigned OpNo, SStream *O)
 
 	if (!MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
 		printOperand(MI, OpNo, O);
+
 		return;
 	}
 
-	imm = MCOperand_getImm(MCInst_getOperand(MI, OpNo)) * 4;
+	imm = SignExtend32(MCOperand_getImm(MCInst_getOperand(MI, OpNo)) * 4, 32);
+	//imm = MCOperand_getImm(MCInst_getOperand(MI, OpNo)) * 4;
 
 	if (!PPC_abs_branch(MI->csh, MCInst_getOpcode(MI))) {
-		imm = MI->address + imm;
+		imm += MI->address;
 	}
 
-	SStream_concat(O, "0x%"PRIx64, imm);
+	printUInt64(O, imm);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -560,13 +1062,9 @@ static void printAbsBranchOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	}
 }
 
-
-#define GET_REGINFO_ENUM
-#include "PPCGenRegisterInfo.inc"
-
 static void printcrbitm(MCInst *MI, unsigned OpNo, SStream *O)
 {
-	unsigned RegNo, tmp;
+	unsigned RegNo;
 	unsigned CCReg = MCOperand_getReg(MCInst_getOperand(MI, OpNo));
 
 	switch (CCReg) {
@@ -581,18 +1079,14 @@ static void printcrbitm(MCInst *MI, unsigned OpNo, SStream *O)
 		case PPC_CR7: RegNo = 7; break;
 	}
 
-	tmp = 0x80 >> RegNo;
-	if (tmp > HEX_THRESHOLD)
-		SStream_concat(O, "0x%x", tmp);
-	else
-		SStream_concat(O, "%u", tmp);
+	printUInt32(O, 0x80 >> RegNo);
 }
 
 static void printMemRegImm(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	set_mem_access(MI, true);
 
-	printS16ImmOperand_Mem(MI, OpNo, O);
+	printS16ImmOperand(MI, OpNo, O);
 
 	SStream_concat0(O, "(");
 
@@ -602,6 +1096,20 @@ static void printMemRegImm(MCInst *MI, unsigned OpNo, SStream *O)
 		printOperand(MI, OpNo + 1, O);
 
 	SStream_concat0(O, ")");
+
+	set_mem_access(MI, false);
+}
+
+static void printPSMemRegImm(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	set_mem_access(MI, true);
+
+	printS12ImmOperand(MI, OpNo, O);
+
+	SStream_concat0(O, "(");
+	printOperand(MI, OpNo + 1, O);
+	SStream_concat0(O, ")");
+
 	set_mem_access(MI, false);
 }
 
@@ -630,13 +1138,13 @@ static void printTLSCall(MCInst *MI, unsigned OpNo, SStream *O)
 	SStream_concat0(O, "(");
 	printOperand(MI, OpNo + 1, O);
 	SStream_concat0(O, ")");
+
 	set_mem_access(MI, false);
 }
 
-#ifndef CAPSTONE_DIET
 /// stripRegisterPrefix - This method strips the character prefix from a
 /// register name so that only the number is left.  Used by for linux asm.
-static const char *stripRegisterPrefix(const char *RegName)
+static char *stripRegisterPrefix(const char *RegName)
 {
 	switch (RegName[0]) {
 		case 'r':
@@ -644,16 +1152,24 @@ static const char *stripRegisterPrefix(const char *RegName)
 		case 'q': // for QPX
 		case 'v':
 			if (RegName[1] == 's')
-				return RegName + 2;
-			return RegName + 1;
+				return cs_strdup(RegName + 2);
+
+			return cs_strdup(RegName + 1);
 		case 'c':
-			if (RegName[1] == 'r')
-				return RegName + 2;
+			if (RegName[1] == 'r') {
+				// skip the first 2 letters "cr"
+				char *name = cs_strdup(RegName + 2);
+
+				// also strip the last 2 letters
+				if(strlen(name) > 2)
+					name[strlen(name) - 2] = '\0';
+
+				return name;
+			}
 	}
 
-	return RegName;
+	return cs_strdup(RegName);
 }
-#endif
 
 static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
@@ -662,15 +1178,19 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 		unsigned reg = MCOperand_getReg(Op);
 #ifndef CAPSTONE_DIET
 		const char *RegName = getRegisterName(reg);
-#endif
-		// map to public register
-		reg = PPC_map_register(reg);
-#ifndef CAPSTONE_DIET
-		// The linux and AIX assembler does not take register prefixes.
-		if (MI->csh->syntax == CS_OPT_SYNTAX_NOREGNAME)
-			RegName = stripRegisterPrefix(RegName);
 
-		SStream_concat0(O, RegName);
+		// printf("reg = %u (%s)\n", reg, RegName);
+
+		// convert internal register ID to public register ID
+		reg = PPC_name_reg(RegName);
+
+		// The linux and AIX assembler does not take register prefixes.
+		if (MI->csh->syntax == CS_OPT_SYNTAX_NOREGNAME) {
+			char *name = stripRegisterPrefix(RegName);
+			SStream_concat0(O, name);
+			cs_mem_free(name);
+		} else
+			SStream_concat0(O, RegName);
 #endif
 
 		if (MI->csh->detail) {
@@ -711,293 +1231,8 @@ static void op_addImm(MCInst *MI, int v)
 	}
 }
 
-static void op_addReg(MCInst *MI, unsigned int reg)
-{
-	if (MI->csh->detail) {
-		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_REG;
-		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].reg = reg;
-		MI->flat_insn->detail->ppc.op_count++;
-	}
-}
-
-static void op_addBC(MCInst *MI, unsigned int bc)
-{
-	if (MI->csh->detail) {
-		MI->flat_insn->detail->ppc.bc = (ppc_bc)bc;
-	}
-}
-
-#define CREQ (0)
-#define CRGT (1)
-#define CRLT (2)
-#define CRUN (3)
-
-static int getBICRCond(int bi)
-{
-	return (bi-PPC_CR0EQ) >> 3;
-}
-
-static int getBICR(int bi)
-{
-	return ((bi - PPC_CR0EQ) & 7) + PPC_CR0;
-}
-
-static char *printAliasInstrEx(MCInst *MI, SStream *OS, void *info)
-{
-#define GETREGCLASS_CONTAIN(_class, _reg) MCRegisterClass_contains(MCRegisterInfo_getRegClass(MRI, _class), MCOperand_getReg(MCInst_getOperand(MI, _reg)))
-	SStream ss;
-	const char *opCode;
-	char *tmp, *AsmMnem, *AsmOps, *c;
-	int OpIdx, PrintMethodIdx;
-	int decCtr = false, needComma = false;
-	MCRegisterInfo *MRI = (MCRegisterInfo *)info;
-
-	SStream_Init(&ss);
-	switch (MCInst_getOpcode(MI)) {
-		default: return NULL;
-		case PPC_gBC:
-				 opCode = "b%s";
-				 break;
-		case PPC_gBCA:
-				 opCode = "b%sa";
-				 break;
-		case PPC_gBCCTR:
-				 opCode = "b%sctr";
-				 break;
-		case PPC_gBCCTRL:
-				 opCode = "b%sctrl";
-				 break;
-		case PPC_gBCL:
-				 opCode = "b%sl";
-				 break;
-		case PPC_gBCLA:
-				 opCode = "b%sla";
-				 break;
-		case PPC_gBCLR:
-				 opCode = "b%slr";
-				 break;
-		case PPC_gBCLRL:
-				 opCode = "b%slrl";
-				 break;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 0) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 1)) {
-		SStream_concat(&ss, opCode, "dnzf");
-		decCtr = true;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 2) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 3)) {
-		SStream_concat(&ss, opCode, "dzf");
-		decCtr = true;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 4) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 7) &&
-			MCOperand_isReg(MCInst_getOperand(MI, 1)) &&
-			GETREGCLASS_CONTAIN(PPC_CRBITRCRegClassID, 1)) {
-		int cr = getBICRCond(MCOperand_getReg(MCInst_getOperand(MI, 1)));
-		switch(cr) {
-			case CREQ:
-				SStream_concat(&ss, opCode, "ne");
-				break;
-			case CRGT:
-				SStream_concat(&ss, opCode, "le");
-				break;
-			case CRLT:
-				SStream_concat(&ss, opCode, "ge");
-				break;
-			case CRUN:
-				SStream_concat(&ss, opCode, "ns");
-				break;
-		}
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 6)
-			SStream_concat0(&ss, "-");
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 7)
-			SStream_concat0(&ss, "+");
-
-		decCtr = false;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 8) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 9)) {
-		SStream_concat(&ss, opCode, "dnzt");
-		decCtr = true;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 10) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 11)) {
-		SStream_concat(&ss, opCode, "dzt");
-		decCtr = true;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) >= 12) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) <= 15) &&
-			MCOperand_isReg(MCInst_getOperand(MI, 1)) &&
-			GETREGCLASS_CONTAIN(PPC_CRBITRCRegClassID, 1)) {
-		int cr = getBICRCond(MCOperand_getReg(MCInst_getOperand(MI, 1)));
-		switch(cr) {
-			case CREQ:
-				SStream_concat(&ss, opCode, "eq");
-				break;
-			case CRGT:
-				SStream_concat(&ss, opCode, "gt");
-				break;
-			case CRLT:
-				SStream_concat(&ss, opCode, "lt");
-				break;
-			case CRUN:
-				SStream_concat(&ss, opCode, "so");
-				break;
-		}
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 14)
-			SStream_concat0(&ss, "-");
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 15)
-			SStream_concat0(&ss, "+");
-
-		decCtr = false;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			((MCOperand_getImm(MCInst_getOperand(MI, 0)) & 0x12)== 16)) {
-		SStream_concat(&ss, opCode, "dnz");
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 24)
-			SStream_concat0(&ss, "-");
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 25)
-			SStream_concat0(&ss, "+");
-
-		needComma = false;
-	}
-
-	if (MCInst_getNumOperands(MI) == 3 &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			((MCOperand_getImm(MCInst_getOperand(MI, 0)) & 0x12)== 18)) {
-		SStream_concat(&ss, opCode, "dz");
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 26)
-			SStream_concat0(&ss, "-");
-
-		if (MCOperand_getImm(MCInst_getOperand(MI, 0)) == 27)
-			SStream_concat0(&ss, "+");
-
-		needComma = false;
-	}
-
-	if (MCOperand_isReg(MCInst_getOperand(MI, 1)) &&
-			GETREGCLASS_CONTAIN(PPC_CRBITRCRegClassID, 1) &&
-			MCOperand_isImm(MCInst_getOperand(MI, 0)) &&
-			(MCOperand_getImm(MCInst_getOperand(MI, 0)) < 16)) {
-		int cr = getBICR(MCOperand_getReg(MCInst_getOperand(MI, 1)));
-
-		if (decCtr) {
-			needComma = true;
-			SStream_concat0(&ss, " ");
-
-			if (cr > PPC_CR0) {
-				SStream_concat(&ss, "4*cr%d+", cr - PPC_CR0);
-			}
-
-			cr = getBICRCond(MCOperand_getReg(MCInst_getOperand(MI, 1)));
-			switch(cr) {
-				case CREQ:
-					SStream_concat0(&ss, "eq");
-					op_addBC(MI, PPC_BC_EQ);
-					break;
-				case CRGT:
-					SStream_concat0(&ss, "gt");
-					op_addBC(MI, PPC_BC_GT);
-					break;
-				case CRLT:
-					SStream_concat0(&ss, "lt");
-					op_addBC(MI, PPC_BC_LT);
-					break;
-				case CRUN:
-					SStream_concat0(&ss, "so");
-					op_addBC(MI, PPC_BC_SO);
-					break;
-			}
-
-			cr = getBICR(MCOperand_getReg(MCInst_getOperand(MI, 1)));
-			if (cr > PPC_CR0) {
-				if (MI->csh->detail) {
-					MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_CRX;
-					MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].crx.scale = 4;
-					MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].crx.reg = PPC_REG_CR0 + cr - PPC_CR0;
-					MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].crx.cond = MI->flat_insn->detail->ppc.bc;
-					MI->flat_insn->detail->ppc.op_count++;
-				}
-			}
-		} else {
-			if (cr > PPC_CR0) {
-				needComma = true;
-				SStream_concat(&ss, " cr%d", cr - PPC_CR0);
-				op_addReg(MI, PPC_REG_CR0 + cr - PPC_CR0);
-			}
-		}
-	}
-
-	if (MCOperand_isImm(MCInst_getOperand(MI, 2)) &&
-			MCOperand_getImm(MCInst_getOperand(MI, 2)) != 0) {
-		if (needComma)
-			SStream_concat0(&ss, ",");
-
-		SStream_concat0(&ss, " $\xFF\x03\x01");
-	}
-
-	tmp = cs_strdup(ss.buffer);
-	AsmMnem = tmp;
-	for(AsmOps = tmp; *AsmOps; AsmOps++) {
-		if (*AsmOps == ' ' || *AsmOps == '\t') {
-			*AsmOps = '\0';
-			AsmOps++;
-			break;
-		}
-	}
-
-	SStream_concat0(OS, AsmMnem);
-	if (*AsmOps) {
-		SStream_concat0(OS, "\t");
-		for (c = AsmOps; *c; c++) {
-			if (*c == '$') {
-				c += 1;
-				if (*c == (char)0xff) {
-					c += 1;
-					OpIdx = *c - 1;
-					c += 1;
-					PrintMethodIdx = *c - 1;
-					printCustomAliasOperand(MI, OpIdx, PrintMethodIdx, OS);
-				} else
-					printOperand(MI, *c - 1, OS);
-			} else {
-				SStream_concat(OS, "%c", *c);
-			}
-		}
-	}
-
-	return tmp;
-}
-
 #define PRINT_ALIAS_INSTR
+#include "PPCGenRegisterName.inc"
 #include "PPCGenAsmWriter.inc"
 
 #endif
