@@ -15,7 +15,7 @@
  *===----------------------------------------------------------------------===*/
 
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */
 
 /*
  * This header file provides those definitions that need to be shared between
@@ -34,7 +34,7 @@
 #define XOP8_MAP_SYM      x86DisassemblerXOP8Opcodes
 #define XOP9_MAP_SYM      x86DisassemblerXOP9Opcodes
 #define XOPA_MAP_SYM      x86DisassemblerXOPAOpcodes
-#define T3DNOW_MAP_SYM    x86DisassemblerT3DNOWOpcodes
+#define THREEDNOW_MAP_SYM x86Disassembler3DNowOpcodes
 
 
 /*
@@ -81,7 +81,6 @@ enum attributeBits {
                                         "operands change width")               \
   ENUM_ENTRY(IC_ADSIZE,             3,  "requires an ADSIZE prefix, so "       \
                                         "operands change width")               \
-  ENUM_ENTRY(IC_OF,                 2,  "requires 0f prefix ")                 \
   ENUM_ENTRY(IC_OPSIZE_ADSIZE,      4,  "requires ADSIZE and OPSIZE prefixes") \
   ENUM_ENTRY(IC_XD,                 2,  "may say something about the opcode "  \
                                         "but not the operands")                \
@@ -91,6 +90,10 @@ enum attributeBits {
                                         "operands change width")               \
   ENUM_ENTRY(IC_XS_OPSIZE,          3,  "requires an OPSIZE prefix, so "       \
                                         "operands change width")               \
+  ENUM_ENTRY(IC_XD_ADSIZE,          3,  "requires an ADSIZE prefix, so "       \
+                                        "operands change width")               \
+  ENUM_ENTRY(IC_XS_ADSIZE,          3,  "requires an ADSIZE prefix, so "       \
+                                        "operands change width")               \
   ENUM_ENTRY(IC_64BIT_REXW,         5,  "requires a REX.W prefix, so operands "\
                                         "change width; overrides IC_OPSIZE")   \
   ENUM_ENTRY(IC_64BIT_REXW_ADSIZE,  6,  "requires a REX.W prefix and 0x67 "    \
@@ -104,6 +107,8 @@ enum attributeBits {
   ENUM_ENTRY(IC_64BIT_XS,           6,  "Just as meaningful as IC_64BIT_XD")   \
   ENUM_ENTRY(IC_64BIT_XD_OPSIZE,    3,  "Just as meaningful as IC_XD_OPSIZE")  \
   ENUM_ENTRY(IC_64BIT_XS_OPSIZE,    3,  "Just as meaningful as IC_XS_OPSIZE")  \
+  ENUM_ENTRY(IC_64BIT_XD_ADSIZE,    3,  "Just as meaningful as IC_XD_ADSIZE")  \
+  ENUM_ENTRY(IC_64BIT_XS_ADSIZE,    3,  "Just as meaningful as IC_XS_ADSIZE")  \
   ENUM_ENTRY(IC_64BIT_REXW_XS,      7,  "OPSIZE could mean a different "       \
                                         "opcode")                              \
   ENUM_ENTRY(IC_64BIT_REXW_XD,      7,  "Just as meaningful as "               \
@@ -272,7 +277,6 @@ enum attributeBits {
   ENUM_ENTRY(IC_EVEX_L2_W_XD_KZ,     4,  "requires EVEX_KZ, L2, W and XD prefix")    \
   ENUM_ENTRY(IC_EVEX_L2_W_OPSIZE_KZ, 4,  "requires EVEX_KZ, L2, W and OpSize")
 
-
 #define ENUM_ENTRY(n, r, d) n,
 typedef enum {
 	INSTRUCTION_CONTEXTS
@@ -292,7 +296,7 @@ typedef enum {
 	XOP8_MAP      = 4,
 	XOP9_MAP      = 5,
 	XOPA_MAP      = 6,
-	T3DNOW_MAP    = 7
+	THREEDNOW_MAP = 7
 } OpcodeType;
 
 /*
@@ -349,6 +353,15 @@ typedef enum {
     case ENCODING_RM_CD32:   \
     case ENCODING_RM_CD64
  
+#define CASE_ENCODING_VSIB   \
+    case ENCODING_VSIB:      \
+    case ENCODING_VSIB_CD2:  \
+    case ENCODING_VSIB_CD4:  \
+    case ENCODING_VSIB_CD8:  \
+    case ENCODING_VSIB_CD16: \
+    case ENCODING_VSIB_CD32: \
+    case ENCODING_VSIB_CD64
+
 // Physical encodings of instruction operands.
 
 #define ENCODINGS                                                            \
@@ -361,14 +374,15 @@ ENUM_ENTRY(ENCODING_RM_CD8, "R/M operand with CDisp scaling of 8")           \
 ENUM_ENTRY(ENCODING_RM_CD16,"R/M operand with CDisp scaling of 16")          \
 ENUM_ENTRY(ENCODING_RM_CD32,"R/M operand with CDisp scaling of 32")          \
 ENUM_ENTRY(ENCODING_RM_CD64,"R/M operand with CDisp scaling of 64")          \
+ENUM_ENTRY(ENCODING_VSIB,     "VSIB operand in ModR/M byte.")                \
+ENUM_ENTRY(ENCODING_VSIB_CD2, "VSIB operand with CDisp scaling of 2")        \
+ENUM_ENTRY(ENCODING_VSIB_CD4, "VSIB operand with CDisp scaling of 4")        \
+ENUM_ENTRY(ENCODING_VSIB_CD8, "VSIB operand with CDisp scaling of 8")        \
+ENUM_ENTRY(ENCODING_VSIB_CD16,"VSIB operand with CDisp scaling of 16")       \
+ENUM_ENTRY(ENCODING_VSIB_CD32,"VSIB operand with CDisp scaling of 32")       \
+ENUM_ENTRY(ENCODING_VSIB_CD64,"VSIB operand with CDisp scaling of 64")       \
 ENUM_ENTRY(ENCODING_VVVV,   "Register operand in VEX.vvvv byte.")            \
 ENUM_ENTRY(ENCODING_WRITEMASK, "Register operand in EVEX.aaa byte.")         \
-ENUM_ENTRY(ENCODING_CB,     "1-byte code offset (possible new CS value)")    \
-ENUM_ENTRY(ENCODING_CW,     "2-byte")                                        \
-ENUM_ENTRY(ENCODING_CD,     "4-byte")                                        \
-ENUM_ENTRY(ENCODING_CP,     "6-byte")                                        \
-ENUM_ENTRY(ENCODING_CO,     "8-byte")                                        \
-ENUM_ENTRY(ENCODING_CT,     "10-byte")                                       \
 ENUM_ENTRY(ENCODING_IB,     "1-byte immediate")                              \
 ENUM_ENTRY(ENCODING_IW,     "2-byte")                                        \
 ENUM_ENTRY(ENCODING_ID,     "4-byte")                                        \
@@ -382,6 +396,7 @@ ENUM_ENTRY(ENCODING_FP,     "Position on floating-point stack in ModR/M "    \
                             "byte.")                                         \
 ENUM_ENTRY(ENCODING_Iv,     "Immediate of operand size")                     \
 ENUM_ENTRY(ENCODING_Ia,     "Immediate of address size")                     \
+ENUM_ENTRY(ENCODING_IRC,    "Immediate for static rounding control")         \
 ENUM_ENTRY(ENCODING_Rv,     "Register code of operand size added to the "    \
                             "opcode byte")                                   \
 ENUM_ENTRY(ENCODING_DUP,    "Duplicate of another operand; ID is encoded "   \
@@ -399,91 +414,43 @@ typedef enum {
 /*
  * Semantic interpretations of instruction operands.
  */
-
 #define TYPES                                                                  \
-ENUM_ENTRY(TYPE_NONE,       "")                                              \
-ENUM_ENTRY(TYPE_REL8,       "1-byte immediate address")                      \
-ENUM_ENTRY(TYPE_REL16,      "2-byte")                                        \
-ENUM_ENTRY(TYPE_REL32,      "4-byte")                                        \
-ENUM_ENTRY(TYPE_REL64,      "8-byte")                                        \
-ENUM_ENTRY(TYPE_PTR1616,    "2+2-byte segment+offset address")               \
-ENUM_ENTRY(TYPE_PTR1632,    "2+4-byte")                                      \
-ENUM_ENTRY(TYPE_PTR1664,    "2+8-byte")                                      \
-ENUM_ENTRY(TYPE_R8,         "1-byte register operand")                       \
-ENUM_ENTRY(TYPE_R16,        "2-byte")                                        \
-ENUM_ENTRY(TYPE_R32,        "4-byte")                                        \
-ENUM_ENTRY(TYPE_R64,        "8-byte")                                        \
-ENUM_ENTRY(TYPE_IMM8,       "1-byte immediate operand")                      \
-ENUM_ENTRY(TYPE_IMM16,      "2-byte")                                        \
-ENUM_ENTRY(TYPE_IMM32,      "4-byte")                                        \
-ENUM_ENTRY(TYPE_IMM64,      "8-byte")                                        \
-ENUM_ENTRY(TYPE_IMM3,       "1-byte immediate operand between 0 and 7")      \
-ENUM_ENTRY(TYPE_IMM5,       "1-byte immediate operand between 0 and 31")     \
-ENUM_ENTRY(TYPE_AVX512ICC,  "1-byte immediate operand for AVX512 icmp")      \
-ENUM_ENTRY(TYPE_UIMM8,      "1-byte unsigned immediate operand")             \
-ENUM_ENTRY(TYPE_RM8,        "1-byte register or memory operand")             \
-ENUM_ENTRY(TYPE_RM16,       "2-byte")                                        \
-ENUM_ENTRY(TYPE_RM32,       "4-byte")                                        \
-ENUM_ENTRY(TYPE_RM64,       "8-byte")                                        \
-ENUM_ENTRY(TYPE_M,          "Memory operand")                                \
-ENUM_ENTRY(TYPE_M8,         "1-byte")                                        \
-ENUM_ENTRY(TYPE_M16,        "2-byte")                                        \
-ENUM_ENTRY(TYPE_M32,        "4-byte")                                        \
-ENUM_ENTRY(TYPE_M64,        "8-byte")                                        \
-ENUM_ENTRY(TYPE_LEA,        "Effective address")                             \
-ENUM_ENTRY(TYPE_M128,       "16-byte (SSE/SSE2)")                            \
-ENUM_ENTRY(TYPE_M256,       "256-byte (AVX)")                                \
-ENUM_ENTRY(TYPE_M1616,      "2+2-byte segment+offset address")               \
-ENUM_ENTRY(TYPE_M1632,      "2+4-byte")                                      \
-ENUM_ENTRY(TYPE_M1664,      "2+8-byte")                                      \
-ENUM_ENTRY(TYPE_SRCIDX8,    "1-byte memory at source index")                 \
-ENUM_ENTRY(TYPE_SRCIDX16,   "2-byte memory at source index")                 \
-ENUM_ENTRY(TYPE_SRCIDX32,   "4-byte memory at source index")                 \
-ENUM_ENTRY(TYPE_SRCIDX64,   "8-byte memory at source index")                 \
-ENUM_ENTRY(TYPE_DSTIDX8,    "1-byte memory at destination index")            \
-ENUM_ENTRY(TYPE_DSTIDX16,   "2-byte memory at destination index")            \
-ENUM_ENTRY(TYPE_DSTIDX32,   "4-byte memory at destination index")            \
-ENUM_ENTRY(TYPE_DSTIDX64,   "8-byte memory at destination index")            \
-ENUM_ENTRY(TYPE_MOFFS8,     "1-byte memory offset (relative to segment "     \
-                            "base)")                                         \
-ENUM_ENTRY(TYPE_MOFFS16,    "2-byte")                                        \
-ENUM_ENTRY(TYPE_MOFFS32,    "4-byte")                                        \
-ENUM_ENTRY(TYPE_MOFFS64,    "8-byte")                                        \
-ENUM_ENTRY(TYPE_SREG,       "Byte with single bit set: 0 = ES, 1 = CS, "     \
-		"2 = SS, 3 = DS, 4 = FS, 5 = GS")                \
-ENUM_ENTRY(TYPE_M32FP,      "32-bit IEE754 memory floating-point operand")   \
-ENUM_ENTRY(TYPE_M64FP,      "64-bit")                                        \
-ENUM_ENTRY(TYPE_M80FP,      "80-bit extended")                               \
-ENUM_ENTRY(TYPE_ST,         "Position on the floating-point stack")          \
-ENUM_ENTRY(TYPE_MM64,       "8-byte MMX register")                           \
-ENUM_ENTRY(TYPE_XMM,        "XMM register operand")                          \
-ENUM_ENTRY(TYPE_XMM32,      "4-byte XMM register or memory operand")         \
-ENUM_ENTRY(TYPE_XMM64,      "8-byte")                                        \
-ENUM_ENTRY(TYPE_XMM128,     "16-byte")                                       \
-ENUM_ENTRY(TYPE_XMM256,     "32-byte")                                       \
-ENUM_ENTRY(TYPE_XMM512,     "64-byte")                                       \
-ENUM_ENTRY(TYPE_VK1,        "1-bit")                                         \
-ENUM_ENTRY(TYPE_VK2,        "2-bit")                                         \
-ENUM_ENTRY(TYPE_VK4,        "4-bit")                                         \
-ENUM_ENTRY(TYPE_VK8,        "8-bit")                                         \
-ENUM_ENTRY(TYPE_VK16,       "16-bit")                                        \
-ENUM_ENTRY(TYPE_VK32,       "32-bit")                                        \
-ENUM_ENTRY(TYPE_VK64,       "64-bit")                                        \
-ENUM_ENTRY(TYPE_XMM0,       "Implicit use of XMM0")                          \
-ENUM_ENTRY(TYPE_SEGMENTREG, "Segment register operand")                      \
-ENUM_ENTRY(TYPE_DEBUGREG,   "Debug register operand")                        \
-ENUM_ENTRY(TYPE_CONTROLREG, "Control register operand")                      \
-\
-ENUM_ENTRY(TYPE_Mv,         "Memory operand of operand size")                \
-ENUM_ENTRY(TYPE_Rv,         "Register operand of operand size")              \
-ENUM_ENTRY(TYPE_IMMv,       "Immediate operand of operand size")             \
-ENUM_ENTRY(TYPE_RELv,       "Immediate address of operand size")             \
-ENUM_ENTRY(TYPE_DUP0,       "Duplicate of operand 0")                        \
-ENUM_ENTRY(TYPE_DUP1,       "operand 1")                                     \
-ENUM_ENTRY(TYPE_DUP2,       "operand 2")                                     \
-ENUM_ENTRY(TYPE_DUP3,       "operand 3")                                     \
-ENUM_ENTRY(TYPE_DUP4,       "operand 4")                                     \
-ENUM_ENTRY(TYPE_M512,       "512-bit FPU/MMX/XMM/MXCSR state")
+  ENUM_ENTRY(TYPE_NONE,       "")                                              \
+  ENUM_ENTRY(TYPE_REL,        "immediate address")                             \
+  ENUM_ENTRY(TYPE_R8,         "1-byte register operand")                       \
+  ENUM_ENTRY(TYPE_R16,        "2-byte")                                        \
+  ENUM_ENTRY(TYPE_R32,        "4-byte")                                        \
+  ENUM_ENTRY(TYPE_R64,        "8-byte")                                        \
+  ENUM_ENTRY(TYPE_IMM,        "immediate operand")                      \
+  ENUM_ENTRY(TYPE_IMM3,       "1-byte immediate operand between 0 and 7")      \
+  ENUM_ENTRY(TYPE_IMM5,       "1-byte immediate operand between 0 and 31")     \
+  ENUM_ENTRY(TYPE_AVX512ICC,  "1-byte immediate operand for AVX512 icmp")      \
+  ENUM_ENTRY(TYPE_UIMM8,      "1-byte unsigned immediate operand")             \
+  ENUM_ENTRY(TYPE_M,          "Memory operand")                                \
+  ENUM_ENTRY(TYPE_MVSIBX,     "Memory operand using XMM index")                \
+  ENUM_ENTRY(TYPE_MVSIBY,     "Memory operand using YMM index")                \
+  ENUM_ENTRY(TYPE_MVSIBZ,     "Memory operand using ZMM index")                \
+  ENUM_ENTRY(TYPE_SRCIDX,     "memory at source index")                        \
+  ENUM_ENTRY(TYPE_DSTIDX,     "memory at destination index")                   \
+  ENUM_ENTRY(TYPE_MOFFS,      "memory offset (relative to segment base)")      \
+  ENUM_ENTRY(TYPE_ST,         "Position on the floating-point stack")          \
+  ENUM_ENTRY(TYPE_MM64,       "8-byte MMX register")                           \
+  ENUM_ENTRY(TYPE_XMM,        "16-byte")                                       \
+  ENUM_ENTRY(TYPE_YMM,        "32-byte")                                       \
+  ENUM_ENTRY(TYPE_ZMM,        "64-byte")                                       \
+  ENUM_ENTRY(TYPE_VK,         "mask register")                                 \
+  ENUM_ENTRY(TYPE_SEGMENTREG, "Segment register operand")                      \
+  ENUM_ENTRY(TYPE_DEBUGREG,   "Debug register operand")                        \
+  ENUM_ENTRY(TYPE_CONTROLREG, "Control register operand")                      \
+  ENUM_ENTRY(TYPE_BNDR,       "MPX bounds register")                           \
+                                                                               \
+  ENUM_ENTRY(TYPE_Rv,         "Register operand of operand size")              \
+  ENUM_ENTRY(TYPE_RELv,       "Immediate address of operand size")             \
+  ENUM_ENTRY(TYPE_DUP0,       "Duplicate of operand 0")                        \
+  ENUM_ENTRY(TYPE_DUP1,       "operand 1")                                     \
+  ENUM_ENTRY(TYPE_DUP2,       "operand 2")                                     \
+  ENUM_ENTRY(TYPE_DUP3,       "operand 3")                                     \
+  ENUM_ENTRY(TYPE_DUP4,       "operand 4")                                     \
 
 #define ENUM_ENTRY(n, d) n,
 typedef enum {
@@ -493,8 +460,7 @@ typedef enum {
 #undef ENUM_ENTRY
 
 /*
- * OperandSpecifier - The specification for how to extract and interpret one
- *   operand.
+ * The specification for how to extract and interpret one operand.
  */
 typedef struct OperandSpecifier {
 	uint8_t encoding;
