@@ -35,7 +35,7 @@ static char *hex_string(unsigned char *str, size_t len)
 	if (!hex_out) { goto Exit; }
 
 	for (i = 0; i < len; ++i) {
-		snprintf(hex_out + (i*2), 2, "%02x", str[i]);
+		snprintf(hex_out + (i * 2), 3, "%02x", str[i]);
 	}
 
 	hex_out[len*2] = 0; // trailing null
@@ -46,8 +46,6 @@ Exit:
 
 static void snprint_insn_detail(char * buf, size_t * cur, size_t * left, cs_insn *ins)
 {
-	size_t used = 0;
-
 #define _this_printf(...) \
 	{ \
 		size_t used = 0; \
@@ -84,10 +82,10 @@ static void snprint_insn_detail(char * buf, size_t * cur, size_t * left, cs_insn
 				break;
 			case ARM_OP_MEM:
 				_this_printf("\t\toperands[%u].type: MEM\n", i);
-				if (op->mem.base != X86_REG_INVALID)
+				if (op->mem.base != ARM_REG_INVALID)
 					_this_printf("\t\t\toperands[%u].mem.base: REG = %s\n",
 							i, cs_reg_name(handle, op->mem.base));
-				if (op->mem.index != X86_REG_INVALID)
+				if (op->mem.index != ARM_REG_INVALID)
 					_this_printf("\t\t\toperands[%u].mem.index: REG = %s\n",
 							i, cs_reg_name(handle, op->mem.index));
 				if (op->mem.scale != 1)
@@ -286,9 +284,7 @@ static void test_valids()
 		}
 	}};
 
-	struct valid_instructions * valid = NULL;
-
-	uint64_t address = 0x1000;
+	struct valid_instructions *valid = NULL;
 	cs_insn *insn;
 	int i;
 	int j;
@@ -323,8 +319,6 @@ static void test_valids()
 			char tmp_buf[2048];
 			size_t left = 2048;
 			size_t cur = 0;
-			size_t used = 0;
-			int success = 0;
 			char * hex_str = NULL;
 
 			struct valid_code * valid_code = NULL;
@@ -336,6 +330,8 @@ static void test_valids()
 					valid->platform_comment, hex_str, valid_code->start_addr, 
 					valid_code->comment, valid_code->expected_out);
 
+			free(hex_str);
+
 			count = cs_disasm(handle,
 					valid_code->code, valid_code->size, 
 					valid_code->start_addr, 0, &insn
