@@ -1,11 +1,12 @@
 /* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013> */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "../myinttypes.h"
 
-#include <capstone.h>
+#include <capstone/platform.h>
+#include <capstone/capstone.h>
 
 static csh handle;
 
@@ -14,10 +15,10 @@ struct platform {
 	cs_mode mode;
 	unsigned char *code;
 	size_t size;
-	char *comment;
+	const char *comment;
 };
 
-static void print_string_hex(char *comment, unsigned char *str, size_t len)
+static void print_string_hex(const char *comment, unsigned char *str, size_t len)
 {
 	unsigned char *c;
 
@@ -33,6 +34,9 @@ static void print_insn_detail(cs_insn *ins)
 {
 	cs_arm64 *arm64;
 	int i;
+	cs_regs regs_read, regs_write;
+	unsigned char regs_read_count, regs_write_count;
+	unsigned char access;
 
 	// detail can be NULL if SKIPDATA option is turned ON
 	if (ins->detail == NULL)
@@ -51,10 +55,15 @@ static void print_insn_detail(cs_insn *ins)
 				printf("\t\toperands[%u].type: REG = %s\n", i, cs_reg_name(handle, op->reg));
 				break;
 			case ARM64_OP_IMM:
-				printf("\t\toperands[%u].type: IMM = 0x%"PRIx64 "\n", i, op->imm);
+				printf("\t\toperands[%u].type: IMM = 0x%" PRIx64 "\n", i, op->imm);
 				break;
 			case ARM64_OP_FP:
+#if defined(_KERNEL_MODE)
+				// Issue #681: Windows kernel does not support formatting float point
+				printf("\t\toperands[%u].type: FP = <float_point_unsupported>\n", i);
+#else
 				printf("\t\toperands[%u].type: FP = %f\n", i, op->fp);
+#endif
 				break;
 			case ARM64_OP_MEM:
 				printf("\t\toperands[%u].type: MEM\n", i);
@@ -89,6 +98,21 @@ static void print_insn_detail(cs_insn *ins)
 				break;
 		}
 
+		access = op->access;
+		switch(access) {
+			default:
+				break;
+			case CS_AC_READ:
+				printf("\t\toperands[%u].access: READ\n", i);
+				break;
+			case CS_AC_WRITE:
+				printf("\t\toperands[%u].access: WRITE\n", i);
+				break;
+			case CS_AC_READ | CS_AC_WRITE:
+				printf("\t\toperands[%u].access: READ | WRITE\n", i);
+				break;
+		}
+
 		if (op->shift.type != ARM64_SFT_INVALID &&
 				op->shift.value)
 			printf("\t\t\tShift: type = %u, value = %u\n",
@@ -100,9 +124,6 @@ static void print_insn_detail(cs_insn *ins)
 		if (op->vas != ARM64_VAS_INVALID)
 			printf("\t\t\tVector Arrangement Specifier: 0x%x\n", op->vas);
 
-		if (op->vess != ARM64_VESS_INVALID)
-			printf("\t\t\tVector Element Size Specifier: %u\n", op->vess);
-
 		if (op->vector_index != -1)
 			printf("\t\t\tVector Index: %u\n", op->vector_index);
 	}
@@ -111,71 +132,56 @@ static void print_insn_detail(cs_insn *ins)
 		printf("\tUpdate-flags: True\n");
 
 	if (arm64->writeback)
-		printf("\tWrite-back: True\n");
+		printf("\tWrite-back: %s\n", arm64->post_index ? "Post" : "Pre");
 
 	if (arm64->cc)
 		printf("\tCode-condition: %u\n", arm64->cc);
 
+	// Print out all registers accessed by this instruction (either implicit or explicit)
+	if (!cs_regs_access(handle, ins,
+				regs_read, &regs_read_count,
+				regs_write, &regs_write_count)) {
+		if (regs_read_count) {
+			printf("\tRegisters read:");
+			for(i = 0; i < regs_read_count; i++) {
+				printf(" %s", cs_reg_name(handle, regs_read[i]));
+			}
+			printf("\n");
+		}
+
+		if (regs_write_count) {
+			printf("\tRegisters modified:");
+			for(i = 0; i < regs_write_count; i++) {
+				printf(" %s", cs_reg_name(handle, regs_write[i]));
+			}
+			printf("\n");
+		}
+	}
+
 	printf("\n");
 }
 
 static void test()
 {
-//#define ARM64_CODE "\xe1\x0b\x40\xb9"	// ldr		w1, [sp, #0x8]
-//#define ARM64_CODE "\x21\x7c\x00\x53"	// lsr	w1, w1, #0x0
-//#define ARM64_CODE "\x21\x7c\x02\x9b"
-//#define ARM64_CODE "\x20\x04\x81\xda"	// csneg	x0, x1, x1, eq | cneg x0, x1, ne
-//#define ARM64_CODE "\x20\x08\x02\x8b"		// add	x0, x1, x2, lsl #2
-
-//#define ARM64_CODE "\x20\xcc\x20\x8b"
-//#define ARM64_CODE "\xe2\x8f\x40\xa9"	// ldp	x2, x3, [sp, #8]
-//#define ARM64_CODE "\x20\x40\x60\x1e"	// fmov d0, d1
-//#define ARM64_CODE "\x20\x7c\x7d\x93"	// sbfiz	x0, x1, #3, #32
-
-//#define ARM64_CODE "\x20\x88\x43\xb3"	// bfxil	x0, x1, #3, #32
-//#define ARM64_CODE "\x01\x71\x08\xd5"	// sys	#0, c7, c1, #0, x1
-//#define ARM64_CODE "\x00\x71\x28\xd5"	// sysl	x0, #0, c7, c1, #0
-
-//#define ARM64_CODE "\x20\xf4\x18\x9e"	// fcvtzs	x0, s1, #3
-//#define ARM64_CODE "\x20\x74\x0b\xd5"	// dc	zva, x0: FIXME: handle as "sys" insn
-//#define ARM64_CODE "\x00\x90\x24\x1e"	// fmov s0, ##10.00000000
-//#define ARM64_CODE "\xe1\x0b\x40\xb9"	// ldr		w1, [sp, #0x8]
-//#define ARM64_CODE "\x20\x78\x62\xf8"	// ldr x0, [x1, x2, lsl #3]
-//#define ARM64_CODE "\x41\x14\x44\xb3"	// bfm	x1, x2, #4, #5
-//#define ARM64_CODE "\x80\x23\x29\xd5"	// sysl	x0, #1, c2, c3, #4
-//#define ARM64_CODE "\x20\x00\x24\x1e"	// fcvtas	w0, s1
-//#define ARM64_CODE "\x41\x04\x40\xd2"	// eor	x1, x2, #0x3
-//#define ARM64_CODE "\x9f\x33\x03\xd5"	// 	dsb	osh
-//#define ARM64_CODE "\x41\x10\x23\x8a"	// bic	x1, x2, x3, lsl #4
-//#define ARM64_CODE "\x16\x41\x3c\xd5"	// mrs	x22, sp_el1
-//#define ARM64_CODE "\x41\x1c\x63\x0e"	// bic	v1.8b, v2.8b, v3.8b
-//#define ARM64_CODE "\x41\xd4\xe3\x6e"	// fabd	v1.2d, v2.2d, v3.2d
-//#define ARM64_CODE "\x20\x8c\x62\x2e"	// cmeq	v0.4h, v1.4h, v2.4h
-//#define ARM64_CODE "\x20\x98\x20\x4e"	// cmeq	v0.16b, v1.16b, #0
-//#define ARM64_CODE "\x20\x2c\x05\x4e"	// smov	x0, v1.b[2]
-//#define ARM64_CODE "\x21\xe4\x00\x2f"	// movi d1, #0xff
-//#define ARM64_CODE "\x60\x78\x08\xd5"	// at	s1e0w, x0	// FIXME: same problem with dc ZVA
-//#define ARM64_CODE "\x20\x00\xa0\xf2"	// movk	x0, #1, lsl #16
-//#define ARM64_CODE "\x20\x08\x00\xb1"	// adds	x0, x1, #0x2
-//#define ARM64_CODE "\x41\x04\x00\x0f"	// movi v1.2s, #0x2
-//#define ARM64_CODE "\x06\x00\x00\x14"	// b 0x44
-//#define ARM64_CODE "\x00\x90\x24\x1e"	// fmov s0, ##10.00000000
-//#define ARM64_CODE "\x5f\x3f\x03\xd5"	// clrex
-//#define ARM64_CODE "\x5f\x3e\x03\xd5"	// clrex #14
-//#define ARM64_CODE "\x20\x00\x02\xab"	// adds	 x0, x1, x2 (alias of adds x0, x1, x2, lsl #0)
-//#define ARM64_CODE "\x20\xf4\x18\x9e"	// fcvtzs	x0, s1, #3
-//#define ARM64_CODE "\x20\xfc\x02\x9b"	// mneg	x0, x1, x2
-//#define ARM64_CODE "\xd0\xb6\x1e\xd5"	// msr	s3_6_c11_c6_6, x16
-
-//#define ARM64_CODE "\x21\x7c\x02\x9b\x21\x7c\x00\x53\x00\x40\x21\x4b\xe1\x0b\x40\xb9\x20\x04\x81\xda\x20\x08\x02\x8b"
-
-//#define ARM64_CODE "\x09\x00\x38\xd5"	// DBarrier
-//#define ARM64_CODE "\x20\xe4\x3d\x0f\xa2\x00\xae\x9e"
-//#define ARM64_CODE "\x9f\x37\x03\xd5\xbf\x33\x03\xd5\xdf\x3f\x03\xd5"	// DBarrier
-//#define ARM64_CODE "\x10\x5b\xe8\x3c"
-//#define ARM64_CODE "\x00\x18\xa0\x5f\xa2\x00\xae\x9e"
-
-#define ARM64_CODE "\x09\x00\x38\xd5\xbf\x40\x00\xd5\x0c\x05\x13\xd5\x20\x50\x02\x0e\x20\xe4\x3d\x0f\x00\x18\xa0\x5f\xa2\x00\xae\x9e\x9f\x37\x03\xd5\xbf\x33\x03\xd5\xdf\x3f\x03\xd5\x21\x7c\x02\x9b\x21\x7c\x00\x53\x00\x40\x21\x4b\xe1\x0b\x40\xb9\x20\x04\x81\xda\x20\x08\x02\x8b\x10\x5b\xe8\x3c"
+#define ARM64_CODE "\x09\x00\x38\xd5" \
+    "\xbf\x40\x00\xd5" \
+    "\x0c\x05\x13\xd5" \
+    "\x20\x50\x02\x0e" \
+    "\x20\xe4\x3d\x0f" \
+	"\x00\x18\xa0\x5f" \
+	"\xa2\x00\xae\x9e" \
+    "\x9f\x37\x03\xd5" \
+	"\xbf\x33\x03\xd5" \
+	"\xdf\x3f\x03\xd5" \
+	"\x21\x7c\x02\x9b" \
+	"\x21\x7c\x00\x53" \
+	"\x00\x40\x21\x4b" \
+	"\xe1\x0b\x40\xb9" \
+	"\x20\x04\x81\xda" \
+	"\x20\x08\x02\x8b" \
+	"\x10\x5b\xe8\x3c" \
+	"\xfd\x7b\xba\xa9" \
+	"\xfd\xc7\x43\xf8"
 
 	struct platform platforms[] = {
 		{
@@ -196,7 +202,7 @@ static void test()
 		cs_err err = cs_open(platforms[i].arch, platforms[i].mode, &handle);
 		if (err) {
 			printf("Failed on cs_open() with error returned: %u\n", err);
-			continue;
+			abort();
 		}
 
 		cs_option(handle, CS_OPT_DETAIL, CS_OPT_ON);
@@ -211,10 +217,10 @@ static void test()
 			printf("Disasm:\n");
 
 			for (j = 0; j < count; j++) {
-				printf("0x%"PRIx64":\t%s\t%s\n", insn[j].address, insn[j].mnemonic, insn[j].op_str);
+				printf("0x%" PRIx64 ":\t%s\t%s\n", insn[j].address, insn[j].mnemonic, insn[j].op_str);
 				print_insn_detail(&insn[j]);
 			}
-			printf("0x%"PRIx64":\n", insn[j-1].address + insn[j-1].size);
+			printf("0x%" PRIx64 ":\n", insn[j-1].address + insn[j-1].size);
 
 			// free memory allocated by cs_disasm()
 			cs_free(insn, count);
@@ -223,6 +229,7 @@ static void test()
 			printf("Platform: %s\n", platforms[i].comment);
 			print_string_hex("Code: ", platforms[i].code, platforms[i].size);
 			printf("ERROR: Failed to disasm given code!\n");
+			abort();
 		}
 
 		printf("\n");
@@ -231,9 +238,27 @@ static void test()
 	}
 }
 
+void test_macros() {
+	assert(CS_AARCH64(_INS_BL) == ARM64_INS_BL);
+	assert(CS_AARCH64pre(CS_ARCH_) == CS_ARCH_ARM64);
+	assert(CS_AARCH64CC(_AL) == ARM64_CC_AL);
+	assert(CS_AARCH64_VL_(16B) == ARM64_VAS_16B);
+	cs_detail detail = { 0 };
+	CS_cs_aarch64() arm64_detail = { 0 };
+	detail.arm64 = arm64_detail;
+	CS_aarch64_op() op = { 0 };
+	detail.CS_aarch64_.operands[0] = op;
+	CS_aarch64_reg() reg = 1;
+	CS_aarch64_cc() cc = ARM64_CC_AL;
+	CS_aarch64_extender() arm64_extender = ARM64_EXT_SXTB;
+	CS_aarch64_shifter() arm64_shifter = ARM64_SFT_LSL;
+	CS_aarch64_vas() arm64_vas = ARM64_VAS_16B;
+}
+
 int main()
 {
 	test();
+	test_macros();
 
 	return 0;
 }
